<?php
/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: GeoIP.php 1917 2005-03-03 15:27:28Z radek $
*/

/**
 * @package    MaxPlugins
 * @author     Radek Maciaszek <radek@awarez.net>
 */

require_once MAX_PATH . '/lib/Max.php';

/**
 *
 * Max_Plugins is an abstract class for factory the specific plugin from specific folder/package
 * for reading the configuration files
 *
 * @static
 */
class Max_Plugins
{
    var $module;
    var $package;
    var $name;
    
    /**
     * Factory the specific plugin from specific package/plugin, eg reporter or geotargeting
     *
     * @static 
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     *
     * @return object          Plugin object or false if any error occured
     *
     */
    function factory($module, $package, $name = null)
    {
        if(!Max_Plugins::includePluginFile($module, $package, $name)) {
            return false;
        }
        if($name === null) {
            $name = $package;
        }
        $className = Max_Plugins::getPluginClassName($module, $package, $name);
        $obj = &new $className;
        $obj->module = $module;
        $obj->package = $package;
        $obj->name = $name;

        return $obj;
    }
    
    /**
     * Factory main module plugin. Method read the module configuration file
     * and return (if exists) main plugin for module (eg. geotargeting
     * has only one plugin)
     *
     * @static 
     * @param string $module     Module name (plugins/module folder name)
     * @param string $configKey  Name of configuration type
     *
     * @return object            Plugin object or false if any error occured
     *
     */
    function factoryPluginByModuleConfig($module, $configKey = 'type', $omit = 'none')
    {
        // read module configuration file
        $conf = Max_Plugins::getConfigByFileName(Max_Plugins::getConfigFileName($module));
        
        // get plugin type from configuration
        if(!isset($conf[$moduleType])) {
            return false;
        } else {
            $packageName = explode(':', $conf[$moduleType]);
            if(count($packageName) > 1) {
                $package = $packageName[0];
                $name = $packageName[1];
            } else {
                $package = $conf[$moduleType];
                $name = null;
            }
        }
        if($package == $omit) {
            // prevent from factory non existings class
            return null;
        }
        
        return Max_Plugins::factory($module, $package, $name);
    }
    
    /**
     * Include file for plugin
     *
     * @static 
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     *
     * @return bool            True on success else false
     *
     */
    function includePluginFile($module, $package, $name = null)
    {
        if($name === null) {
            $name = $package;
        }
        $fileName = MAX_PATH . "/plugins/{$module}/{$package}/{$name}.plugin.php";
        if(!file_exists($fileName)) {
            MAX::raiseError("Unable to include the {$fileName} file");
            return false;
        } else {
            // TODO: add 'production' option to config and add @ before include_once below
            include_once $fileName;
        }

        $className = Max_Plugins::getPluginClassName($module, $package, $name);
        if (!class_exists($className)) {
            MAX::raiseError("Plugin file included but class '$className' doesn't exists");
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * This method include plugin file and call a static method in plugin class
     *
     * @static 
     * @param string $module        Module name (plugins/module folder name)
     * @param string $package       Package name (plugins/module/package folder namer)
     * @param string $name          Name of the php file which contains the plugin
     * @param string $staticMethod  Name of the php file which contains the plugin
     *
     * @return object          Plugin object or false if any error occured
     *
     */
    function callStaticMethod($module, $package, $name = null, $staticMethod)
    {
        if($name === null) {
            $name = $package;
        }
        if(!Max_Plugins::includePluginFile($module, $package, $name)) {
            return false;
        }
        
        $className = Max_Plugins::getPluginClassName($module, $package, $name);
        $ret = call_user_func(array($className, $staticMethod));
        
        return $ret;
    }
    
    /**
     * Common plugin method for reading the specific plugin config file
     *
     * @param string $module                Module name (plugins/module folder name)
     * @param string $package               Package name (plugins/module/package folder namer)
     * @param string $name                  Name of the php file which contains the plugin
     * @param boolean $processSections      Look at: http://uk.php.net/manual/en/function.parse-ini-file.php
     *                                      as one dimension array
     * @param boolean $commonPackageConfig  If true read the global plugin.conf.ini file
     *                                      for specific package
     *
     * @return array                        Config array
     *
     */
    function staticGetConfig($module, $package, $name = null, $commonPackageConfig = true, $processSections = false)
    {
        if($commonPackageConfig) {
            $configFileName = MAX_PATH . "/plugins/{$module}/{$package}/plugin.conf.ini";
        } else {
            $configFileName = MAX_PATH . "/plugins/{$module}/{$package}/{$name}.plugin.conf.ini";
        }
        return Max_Plugins::getConfigByFileName($configFileName, $processSections);
    }
    
    /**
     * Common plugin method for reading the specific plugin config file
     *
     * @static 
     * @param string $fileName          Module name (plugins/module folder name)
     * @param boolean $processSections  Look at: http://uk.php.net/manual/en/function.parse-ini-file.php
     *
     * @return array                    Config array
     *
     */
    function getConfigByFileName($fileName, $processSections = false)
    {
        if (file_exists($fileName)) {
            $conf = parse_ini_file($fileName, $processSections);
            if(is_array($conf)) {
                return $conf;
            } else {
                return false;
            }
        } else {
            MAX::raiseError("Config file '{$fileName}' doesn't exists", MAX_ERROR_NOFILE);
            return false;
        }
    }
    
    /**
     * This method build and return configuration file path for specific plugin/package/module
     *
     * @static 
     * @param string $module   Module name
     * @param string $package  Package name
     * @param string $name     Plugin name
     *
     * @return string          Path to the configuration file
     *
     */
    function getConfigFileName($module, $package = null, $name = null)
    {
        if($package === null) {
            return MAX_PATH . '/plugins/'.$module.'/plugin.conf.ini';
        } elseif($name === null) {
            return MAX_PATH . '/plugins/'.$module.'/'.$package.'/plugin.conf.ini';
        } else {
            return MAX_PATH . '/plugins/'.$module.'/'.$package.'/'.$name.'.plugin.conf.ini';
        }
    }
    
    /**
     * This method build and return class name for specific plugin
     *
     * @static 
     * @param string $module   Module name
     * @param string $package  Package name
     * @param string $name     Plugin name
     *
     * @return string          Plugin class name
     *
     */
    function getPluginClassName($module, $package, $name = null)
    {
        if($name === null) {
            $name = $package;
        }
        $className = "Plugins_" . ucfirst($module) . '_' . ucfirst($package) . '_' . ucfirst($name);
        
        return $className;
    }
    
    /**
     * Return the list of files from selected folder which match the specified criteria (file mask)
     *
     * @static 
     * @param string $folder      Folder path
     * @param boolean $recursive  If true read all the files also from subdirectory
     *
     * @return array              List of files
     */
    function &getPlugins($module, $package = null, $onlyPluginNameAsIndex = true, $recursive = 1)
    {
        $plugins = array();
        $pluginFiles = Max_Plugins::getPluginsFiles($module, $package, $recursive);
        foreach ($pluginFiles as $key => $pluginFile) {
            $pluginInfo = explode(':', $key);
            if(count($pluginInfo) > 1) {
                if($onlyPluginNameAsIndex) {
                    $plugins[$pluginInfo[1]] = Max_Plugins::factory($module, $pluginInfo[0], $pluginInfo[1]);
                } else {
                    $plugins[$key] = Max_Plugins::factory($module, $pluginInfo[0], $pluginInfo[1]);
                }
            }
        }
        return $plugins;
    }
    
    /**
     * Build a plugin folder path and return the plugin files list from it.
     *
     * @param string $module      Name of the module
     * @param string $package     Name of the package
     * @param boolean $recursive  Search plugins in subdirectories
     *
     * @return array              List of files
     */
    function getPluginsFiles($module, $package = null, $recursive = 1)
    {
        $pluginsDir = MAX_PATH.'/plugins';
        if(!empty($package)) {
            $dir = $pluginsDir.'/'.$module.'/'.$package;
        } else {
            $dir = $pluginsDir.'/'.$module;
        }
        return Max_Plugins::getPluginsFilesFromFolder($dir, $recursive);
    }
    
    /**
     * Return the list of files from selected folder which match the specified criteria (file mask)
     *
     * @static 
     * @param string $folder              Folder path
     * @param integer|boolean $recursive  If true read all the files also from subdirectory
     *                                    If integer read as many subdirectories
     *
     * @return array                      List of files
     */
    function getPluginsFilesFromFolder($folder, $recursive = 1)
    {
        include_once MAX_PATH . '/lib/max/Plugins/FileScanner.php';
        
        $fs = new Max_Plugins_FileScanner();
        $fs->addFileTypes(array('php','inc'));
        $fs->setFileMask('^[a-zA-Z0-9\-_\.\\:/]*/([a-zA-Z0-9\-_]*)/([a-zA-Z0-9\-_]*)\.plugin\.php$');
        $fs->addDir($folder, $recursive);
        
        return $fs->getAllFiles();
    }

    /**
     * Common method to run one method on group of plugins.
     * 
     * TODO: add a possibility to pass additional parameters to $callbackMethod
     *
     * @static 
     * @param array $plugins          Array of plugins
     * @param string $callbackMethod  This method will be executed on every plugin
     *
     * @return array                  Array of results after calling $callbackMethod
     *                                on every plugin
     */
    function &callOnPlugins(&$plugins, $callbackMethod)
    {
        if(!is_array($plugins)) {
            MAX::raiseError('Bad argument', MAX_ERROR_INVALIDARGS);
        }
        $ret = array();
        foreach ($plugins as $key => $plugin) {
            if (is_callable(array($plugin, $callbackMethod))) {
                $ret[$key] = call_user_func(array($plugins[$key], $callbackMethod));
            }
        }
        return $ret;
    }

}

?>